﻿PAF = {
    flash = true,
    voice = true,
    sfx = false,
    text = true,
    dmg = false
}

SLASH_PAF1, SLASH_PAF2 = "/paf", "/procandflash"
local events = {}
local procTextAlpha, flashFrameAlpha = 0, 0
local playerClass = select(2, UnitClass("player"))

local colors = {
    black = {0, 0, 0},
    bloodRed = {188, 0, 0},
    coolBlue = {0, 153, 255},
    coolGray = {85, 85, 100},
    darkPurple = {76, 0, 102},
    darkTeal = {0, 128, 135},
    darkYellow = {192, 145, 0},
    gold = {255, 219, 0},
    green = {0, 192, 0},
    lightBlue = {192, 216, 255},
    orange = {255, 89, 0},
    plagueYellow = {192, 196, 0},
    purple = {192, 0, 255},
    white = {255, 255, 255}
}

local Proc, procList = {}, {}
Proc.__index = Proc

function Proc.add(spellId, color, name, command, nameText, stackDetect)
    local proc = {}
    setmetatable(proc, Proc)
    proc.spell = spellId
    proc.color = {colors[color][1] / 255, colors[color][2] / 255, colors[color][3] / 255}
    proc.name = name
    proc.command = command
    proc.nameText = nameText
    proc.stackDetect = stackDetect
    PAF[command] = true
	if spellId ~= 0 then
		procList[#procList + 1] = proc
	end
    return proc
end

function Proc:stack()
	local stack = select(4, UnitAura("player", GetSpellInfo(self.spell), nil, "HELPFUL"))
    return stack and stack > 0 and stack or 1
end

function Proc:flash()
	if PAF.flash then
		flashFrameAlpha = 1.4
		PAF_FlashTexture:SetTexture(self.color[1], self.color[2], self.color[3], 0.6)
		PAF_FlashFrame:SetAlpha(0.9)
		PAF_FlashFrame:Show()
	end
	if PAF.text then
		procTextAlpha = 2
		PAF_ProcText:SetAlpha(1)
		PAF_ProcText:SetText(self.nameText)
		PAF_ProcText:Show()
	end
	if PAF.voice then
		PlaySoundFile("Interface\\AddOns\\Proc And Flash\\sounds\\" .. self.command .. ".mp3", "MASTER")
	end
	if PAF.sfx then
		PlaySoundFile("Sound\\Spells\\Rogue_shadowdance_state.Wav", "MASTER");
	end
end

if playerClass == "DEATHKNIGHT" then
	Proc.add(59052, "coolBlue", "Freezing Fog (Rime talent)", "freezingfog", "FREEZING FOG", -1)
	Proc.add(51124, "darkPurple", "Killing Machine talent", "killingmachine", "KILLING MACHINE", 1)
	Proc.add(81340, "green", "Sudden Doom talent", "suddendoom", "SUDDEN DOOM", -1)
elseif playerClass == "DRUID" then

elseif playerClass == "HUNTER" then
	Proc.add(56453, "orange", "Lock and Load talent", "lockandload", "LOCK AND LOAD", 2)
	Proc.add(82926, "white", "Master Marksman talent", "mastermarksman", "MASTER MARKSMAN", 1)
elseif playerClass == "MAGE" then
	Proc.add(31643, "white", "Blazing Speed talent", "blazingspeed", "BLAZING SPEED", 1)
	Proc.add(57761, "lightBlue", "Brain Freeze talent", "brainfreeze", "BRAIN FREEZE", 1)
	Proc.add(44544, "coolBlue", "Fingers of Frost talent", "fingersoffrost", "FINGERS OF FROST", -1)
	Proc.add(48108, "orange", "Hot Streak talent", "hotstreak", "HOT STREAK", 1)
	Proc.add(64343, "bloodRed", "Impact talent", "impact", "IMPACT", 1)
elseif playerClass == "PALADIN" then
	Proc.add(59578, "gold", "The Art of War talent", "artofwar", "THE ART OF WAR", 1)
elseif playerClass == "PRIEST" then
	Proc.add(88688, "gold", "Surge of Light talent", "surgeoflight", "SURGE OF LIGHT", 1)
elseif playerClass == "ROGUE" then
	Proc.add(109949, "bloodRed", "Fury of the Destroyer (Fangs of the Father)", "fof", "FINISH HIM", 1)
elseif playerClass == "SHAMAN" then
	Proc.add(324, "coolBlue", "Fulmination talent (7 stacks of Lightning Shield, change value in Proc And Flash.lua)", "fulmination", "FULMINATION", 7)
	Proc.add(53817, "lightBlue", "Maelstrom Weapon talent (5 stacks)", "maelstromweapon", "MAELSTROM WEAPON", 5)
elseif playerClass == "WARLOCK" then
	Proc.add(34936, "bloodRed", "Backlash talent", "backlash", "BACKLASH", 1)
	Proc.add(63167, "darkPurple", "Decimation talent", "decimation", "DECIMATION", 1)
	Proc.add(47283, "orange", "Empowered Imp talent", "empoweredimp", "EMPOWERED IMP", 1)
	Proc.add(71165, "orange", "Molten Core talent", "moltencore", "MOLTEN CORE", 3)
	Proc.add(17941, "darkPurple", "Shadow Trance (Nightfall talent)", "shadowtrance", "SHADOW TRANCE", 1)
elseif playerClass == "WARRIOR" then
	Proc.add(12964, "white", "Battle Trance talent", "battletrance", "BATTLE TRANCE", 1)
	Proc.add(46916, "bloodRed", "Bloodsurge talent", "bloodsurge", "BLOODSURGE", -1)
	Proc.add(86627, "darkPurple", "Incite talent", "incite", "INCITE", 1)
	Proc.add(52437, "bloodRed", "Sudden Death talent", "suddendeath", "SUDDEN DEATH", 1)
	Proc.add(50227, "white", "Sword and Board talent", "swordandboard", "SWORD AND BOARD", 1)
end

local function SetDamageFont()
    DAMAGE_TEXT_FONT = "Interface\\AddOns\\Proc And Flash\\font.ttf"
    COMBAT_TEXT_HEIGHT = 25
    COMBAT_TEXT_CRIT_MINHEIGHT = 26
    COMBAT_TEXT_CRIT_MAXHEIGHT = 30
end

local function PrintSetting(setting, on)
    if on then
        print("Proc And Flash - " .. setting .. ": |cFF00C000On")
    else
        print("Proc And Flash - " .. setting .. ": |cFFC00000Off")
    end
end

function SlashCmdList.PAF(msg, editbox)
    msg = { strsplit(" ", strlower(msg)) }
    if (msg[1] == "flash") then
        if msg[2] then PAF.flash = msg[2] == "on" end
        PrintSetting("Flash alert", PAF.flash)
    elseif (msg[1] == "voice") then
        if msg[2] then PAF.voice = msg[2] == "on" end
        PrintSetting("Voice alert", PAF.voice)
    elseif (msg[1] == "sfx") then
        if msg[2] then PAF.sfx = msg[2] == "on" end
        PrintSetting("Sound effect alert", PAF.sfx)
    elseif (msg[1] == "text") then
        if msg[2] then PAF.text = msg[2] == "on" end
        PrintSetting("Text alert", PAF.text)
    elseif (msg[1] == "dmg") then
        if msg[2] then PAF.dmg = msg[2] == "on" end
        PrintSetting("Use proc font as damage font (requires logout)", PAF.dmg)
    elseif (msg[1] == "procs") then
        print("Proc And Flash - Procs available for your class:")
        for i = 1, #procList do
            print("  /paf " .. procList[i].command .. " |cFF00C000on|r/|cFFC00000off|r - enable/disable " .. procList[i].name)
        end
	elseif (msg[1] == "test") then
        for i = 1, #procList do
			if procList[i].command == msg[2] then
				procList[i]:flash()
				return
			end
        end
    elseif (msg[1] == "") then
        print("Proc And Flash (version: |cFFFFD000" .. GetAddOnMetadata("Proc and Flash", "Version") .. "|r) - Commands:")
        print("  /paf flash |cFF00C000on|r/|cFFC00000off|r - enable/disable the flash alerts")
        print("  /paf voice |cFF00C000on|r/|cFFC00000off|r - enable/disable the voice alerts")
        print("  /paf sfx |cFF00C000on|r/|cFFC00000off|r - enable/disable the sound effect alerts")
        print("  /paf text |cFF00C000on|r/|cFFC00000off|r - enable/disable the text alerts")
        print("  /paf dmg |cFF00C000on|r/|cFFC00000off|r - enable/disable using proc font as damage font (requires logout)")
		print("  /paf test |cFFFFD000[proc]|r - test what happens when a specific proc occurs")
        print("  /paf procs - list procs available for your class")
        print("Have more ideas for procs to flash? Visit |cFFFF569Spyware|r on US-Bleeding Hollow Horde or Proc And Flash on |cFFFFD000wowinterface.com|r or |cFFFFD000curse.com|r and suggest!")
    else
        for i = 1, #procList do
			if procList[i].command == msg[1] then
				if msg[2] then PAF[procList[i].command] = msg[2] == "on" end
				PrintSetting(procList[i].name, PAF[procList[i].command])
				return
			end
        end
        print("Proc and Flash - |cFFC00000Invalid command")
    end
end

PAF_HandlerFrame:SetScript("OnUpdate", function(self, elapsed)
    if PAF_FlashFrame:IsShown() then
        if (flashFrameAlpha <= 0.04) then
            PAF_FlashFrame:Hide()
        else
            flashFrameAlpha = flashFrameAlpha - elapsed
            if (flashFrameAlpha <= 0.9) then
                PAF_FlashFrame:SetAlpha(flashFrameAlpha)
            end
        end
    end
    if PAF_ProcText:IsShown() then
        if (procTextAlpha <= 0.04) then
            PAF_ProcText:Hide()
            PAF_ProcText:SetText(nil)
        else
            procTextAlpha = procTextAlpha - elapsed
            if (procTextAlpha <= 1) then
                PAF_ProcText:SetAlpha(procTextAlpha)
            end
        end
    end
end)

function events:ADDON_LOADED(name)
	if name == "Proc And Flash" and PAF.dmg then
		SetDamageFont()
	end
end

function events:COMBAT_LOG_EVENT_UNFILTERED(self, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId)
	if dstGUID == UnitGUID("player") and (eventType == "SPELL_AURA_REFRESH" or eventType == "SPELL_AURA_APPLIED") then
		for i = 1, #procList do
			if procList[i].spell == spellId and PAF[procList[i].command] and (procList[i]:stack() == procList[i].stackDetect or procList[i].stackDetect < 0) then
				procList[i]:flash()
				return
			end
		end
	end
end

PAF_HandlerFrame:SetScript("OnEvent", function(self, event, ...) events[event](self, ...) end)
for k in pairs(events) do
    PAF_HandlerFrame:RegisterEvent(k)
end